<?php
/**
 * @copyright   Copyright (c) http://www.manadev.com
 * @license     http://www.manadev.com/license  Proprietary License
 */

namespace Manadev\ProductCollection\Resources\Facets\Dropdown;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\DB\Select;
use Magento\Framework\Model\ResourceModel\Db;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterface;
use Magento\InventorySalesApi\Api\Data\SalesChannelInterfaceFactory;
use Magento\InventorySalesApi\Api\GetStockBySalesChannelInterface;
use Magento\Store\Model\StoreManagerInterface;
use Manadev\ProductCollection\Configuration;
use Manadev\ProductCollection\Contracts\Facet;
use Manadev\ProductCollection\Contracts\FacetResource;
use Manadev\ProductCollection\Facets\Dropdown\OptimizedFacet;
use Manadev\ProductCollection\Factory;
use Manadev\ProductCollection\Resources\HelperResource;
use Zend_Db;
use Zend_Db_Expr;
use Manadev\ProductCollection\Helper\Data;

class OptimizedFacetResource extends FacetResource
{

    /**
     * @var Data
     */
    protected $helper;

    protected $product_entity_ids_to_filter = [];

    public function __construct(
        Db\Context $context ,
        Factory $factory ,
        StoreManagerInterface $storeManager ,
        Configuration $configuration ,
        HelperResource $helperResource ,
        Data $helper,
        $resourcePrefix = null
    )
    {
        parent::__construct($context , $factory , $storeManager , $configuration , $helperResource , $resourcePrefix);
        $this->helper = $helper;
    }

    /**
     * @param Select $select
     * @param Facet $facet
     * @return mixed
     */
    public function count(Select $select, Facet $facet)
    {
        /* @var $facet OptimizedFacet */
        $this->prepareSelect($select, $facet);
        $result = $this->getConnection()->fetchAll($select);
        return count($result) ? $result : false;
    }

    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     * @return Select|void
     */
    public function prepareSelect(Select $select, OptimizedFacet $facet)
    {
        $this->helperResource->clearFacetSelect($select);

        $selectedOptionIds = $facet->getSelectedOptionIds();

        if ($selectedOptionIds !== false) {
            return $this->_prepareSelectWithOptionIds($select, $facet, $selectedOptionIds);
        }

        return $this->_prepareSelect($select, $facet);
    }

    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     * @param $selectedOptionIds
     */
    private function _prepareSelectWithOptionIds(Select $select, OptimizedFacet $facet, $selectedOptionIds)
    {
        $db = $this->getConnection();
        $fields = [
            'value' => new Zend_Db_Expr("DISTINCT(`eav`.`value`)"),
            'label' => new Zend_Db_Expr("COALESCE(`vs`.`value`, `vg`.`value`)"),
            'count' => new Zend_Db_Expr("1"),
            'sort_order' => new Zend_Db_Expr("`o`.`sort_order`"),
            'is_selected' => new Zend_Db_Expr("`eav`.`value` IN (" . implode(',', $selectedOptionIds). ")")
        ];
        $select
            ->joinInner(['eav' => $this->getTable('catalog_product_index_eav')],
                "`eav`.`entity_id` = `e`.`entity_id` AND
                {$db->quoteInto("`eav`.`attribute_id` = ?", $facet->getAttributeId())} AND
                {$db->quoteInto("`eav`.`store_id` = ?", $this->getStoreId())}", null)
            ->joinInner(['o' => $this->getTable('eav_attribute_option')],
                "`o`.`option_id` = `eav`.`value`", null)
            ->joinInner(['vg' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vg`.`option_id` = `eav`.`value` AND `vg`.`store_id` = ?", 0), null)
            ->joinLeft(['vs' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vs`.`option_id` = `eav`.`value` AND `vs`.`store_id` = ?", $this->getStoreId()), null)
            ->columns($fields)->order('sort_order');


        $inStock = $this->helper->getInStock($facet);

        if (!empty($inStock)) {
            $ids = join(',', $inStock);
            $select->where("`eav`.`value` IN ({$ids})");
        }
        $select->order('sort_order');

        return $select;
    }

    public function getProductEntityIdsToFilter()
    {
        return array_unique($this->helper->getProductEntityIdsToFilter());
    }

    public function setProductEntityIdsToFilter($array = array())
    {
        $this->product_entity_ids_to_filter = $array;
    }



    /**
     * @param Select $select
     * @param OptimizedFacet $facet
     * @return Select
     */
    private function _prepareSelect(Select $select, OptimizedFacet $facet)
    {
        $db = $this->getConnection();
        $fields = [
            'value' => new Zend_Db_Expr("DISTINCT(`eav`.`value`)"),
            'label' => new Zend_Db_Expr("COALESCE(`vs`.`value`, `vg`.`value`)"),
            'count' => new Zend_Db_Expr("1"),
            'sort_order' => new Zend_Db_Expr("`o`.`sort_order`"),
            'is_selected' => new Zend_Db_Expr("0")
        ];
        $select
            ->joinInner(['eav' => $this->getTable('catalog_product_index_eav')],
                "`eav`.`entity_id` = `e`.`entity_id` AND
                {$db->quoteInto("`eav`.`attribute_id` = ?", $facet->getAttributeId())} AND
                {$db->quoteInto("`eav`.`store_id` = ?", $this->getStoreId())}", null)
            ->joinInner(['o' => $this->getTable('eav_attribute_option')],
                "`o`.`option_id` = `eav`.`value`", null)
            ->joinInner(['vg' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vg`.`option_id` = `eav`.`value` AND `vg`.`store_id` = ?", 0), null)
            ->joinLeft(['vs' => $this->getTable('eav_attribute_option_value')],
                $db->quoteInto("`vs`.`option_id` = `eav`.`value` AND `vs`.`store_id` = ?", $this->getStoreId()), null)
            ->columns($fields)->order('sort_order');

        return $select;
    }

    public function getFilterCallback(Facet $facet)
    {
        return $this->helperResource->dontApplyFilterNamed($facet->getName());
    }

    /**
     * Resource initialization
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_setMainTable('catalog_product_index_eav');
    }
}